@@ -2283,7 +2283,7 @@ ATTRIBUTE_MAPPING = {
 "role": {
     "oid": "1.3.6.1.4.1.36560.42.1",
     "display_name": "Role",
-    "profile_field_name": 'roles',
+    "profile_field_name": 'groups',
     "type": "http://www.w3.org/2001/XMLSchema#string",
     "syntax": "1.3.6.1.4.1.1466.115.121.1.15{128}",
 },
