@@ -66,8 +66,28 @@ def get_attributes(user, definitions=None, source=None, auth_source=False, **kwa
                 source.name)
         return None
 
-    attributes = dict()
-    data = []
+    data = {}
+    if hasattr(user, 'get_attributes') and callable(user.get_attributes):
+        attributes = user.get_attributes()
+        if not definitions:
+            definitions = []
+            for key in attributes:
+                if is_definition(key):
+                    definitions.add(key)
+                else:
+                    definition = get_def_name_from_alias(key)
+                    if definition:
+                        definitions.add(definition)
+        for definition in definitions:
+            new = []
+            for key in get_aliases(definition):
+                if key in attributes:
+                    new.append(set(attributes[key]))
+            if not new:
+                continue
+            new = reduce(__or__, new)
+            old = data.get(definition, [])
+            data[definition] = list(set(old) | set(new))
     field_names = set()
     user_profile_fields = getattr(user, 'USER_PROFILE', [])
     if not user_profile_fields:
@@ -104,6 +124,8 @@ def get_attributes(user, definitions=None, source=None, auth_source=False, **kwa
         if value:
             if callable(value):
                 value = value()
+            if hasattr(value, 'all') and callable(value.all):
+                value = value.all()
             logger.debug('field %r has value %r', field_name, value)
             old = data.get(definition, [])
             if not isinstance(value, basestring) and hasattr(value,
@@ -111,30 +133,10 @@ def get_attributes(user, definitions=None, source=None, auth_source=False, **kwa
                 new = map(unicode, value)
             else:
                 new = [unicode(value)]
-            data[definition] = list(set(old) | set(new))
+            if not old:
+                data[definition] = new
         else:
             logger.debug('get_attributes: no value found')
-    if hasattr(user, 'get_attributes') and callable(user.get_attributes):
-        attributes = user.get_attributes()
-        if not definitions:
-            definitions = []
-            for key in attributes:
-                if is_definition(key):
-                    definitions.add(key)
-                else:
-                    definition = get_def_name_from_alias(key)
-                    if definition:
-                        definitions.add(definition)
-        for definition in definitions:
-            new = []
-            for key in get_aliases(definition):
-                if key in attributes:
-                    new.append(set(attributes[key]))
-            if not new:
-                continue
-            new = reduce(__or__, new)
-            old = data.get(definition, [])
-            data[definition] = list(set(old) | set(new))
     data = [{'definition': definition, 'values': values} for definition, values in 
             data.items()]
     return {SOURCE_NAME: data}
