@@ -6,8 +6,7 @@ from django_authopenid.forms import OpenidSigninForm
 from django_authopenid.utils import *
 from django.core.urlresolvers import reverse
 
-import authentic2.auth2_auth.models as models
-from authentic2.auth2_auth import NONCE_FIELD_NAME
+from authentic2.constants import NONCE_FIELD_NAME
 from views import ask_openid, signin_failure
 import views
 
@@ -39,5 +38,5 @@ class OpenIDFrontend(object):
     def template(self):
         return 'auth/login_form_openid.html'
 
-    def profile(self, request, next=''):
-        return views.openid_profile(request, next)
+    def profile(self, request):
+        return views.profile(request)
