@@ -3,10 +3,15 @@ import urllib
 from django.utils.translation import gettext_noop
 from django.http import HttpResponseRedirect
 from django.contrib.auth import REDIRECT_FIELD_NAME
-from authentic2.auth2_auth import NONCE_FIELD_NAME
-from authentic2.auth2_auth.auth2_ssl.login_ssl import *
 import django.forms
 
+
+from authentic2.constants import NONCE_FIELD_NAME
+
+
+from . import views
+
+
 class SSLFrontend(object):
     def enabled(self):
         return True
@@ -31,5 +36,5 @@ class SSLFrontend(object):
     def template(self):
         return 'auth/login_form_ssl.html'
 
-    def profile(self, request, next=''):
-        return profile(request, next)
+    def profile(self, request):
+        return views.profile(request)
