@@ -6,6 +6,8 @@ except ImportError:
     from django.contrib.auth.models import User
     get_user_model = lambda: User
 
+from . import app_settings
+
 user_model_label = getattr(settings, 'AUTH_USER_MODEL', 'auth.User')
 
 use_attribute_aggregator = 'authentic2.attribute_aggregator' \
@@ -18,7 +20,8 @@ def get_registration_fields():
     User = get_user_model()
     username_field = getattr(User, 'USERNAME_FIELD', 'username')
     field_names = getattr(User, 'REGISTER_FIELDS', get_required_fields())
-    return [username_field] + list(field_names)
+    setting_fields = app_settings.A2_REGISTRATION_FIELDS
+    return [username_field] + list(field_names) + list(setting_fields)
 
 def get_required_fields():
     """
@@ -27,4 +30,5 @@ def get_required_fields():
     User = get_user_model()
     username_field = getattr(User, 'USERNAME_FIELD', 'username')
     field_names = getattr(User, 'REQUIRED_FIELDS', [])
-    return [username_field] + list(field_names)
+    setting_fields = app_settings.A2_REGISTRATION_REQUIRED_FIELDS
+    return [username_field] + list(field_names) + list(setting_fields)
