@@ -32,22 +32,22 @@ from django.http import HttpResponse, HttpResponseRedirect, \
     HttpResponseForbidden, HttpResponseBadRequest, Http404
 from django.utils.translation import ugettext as _
 from django.views.decorators.csrf import csrf_exempt
+from django.views.decorators.cache import never_cache
 from django.contrib.auth import BACKEND_SESSION_KEY
 from django.conf import settings
 from django.utils.encoding import smart_unicode
 from django.contrib.auth import load_backend
+from django.shortcuts import render
 
 
-from authentic2.idp.utils import get_username
 from authentic2.compat import get_user_model
-import authentic2.idp as idp
-import authentic2.idp.views as idp_views
+import authentic2.views as a2_views
 from authentic2.idp.models import get_attribute_policy
-from authentic2.saml.models import LibertyAssertion, LibertyArtifact, \
-    LibertySession, LibertyFederation, LibertySessionDump, \
-    nameid2kwargs, saml2_urn_to_nidformat, LIBERTY_SESSION_DUMP_KIND_SP, \
-    nidformat_to_saml2_urn, save_key_values, get_and_delete_key_values, \
-    LibertyProvider, LibertyServiceProvider, NAME_ID_FORMATS
+from authentic2.saml.models import (LibertyArtifact,
+    LibertySession, LibertyFederation, 
+    nameid2kwargs, saml2_urn_to_nidformat,
+    nidformat_to_saml2_urn, save_key_values, get_and_delete_key_values,
+    LibertyProvider, LibertyServiceProvider, SAMLAttribute, NAME_ID_FORMATS)
 from authentic2.saml.common import redirect_next, asynchronous_bindings, \
     soap_bindings, load_provider, get_saml2_request_message, \
     error_page, set_saml2_response_responder_status_code, \
@@ -63,19 +63,20 @@ from authentic2.saml.common import redirect_next, asynchronous_bindings, \
     AUTHENTIC_STATUS_CODE_UNAUTHORIZED, \
     send_soap_request, get_saml2_query_request, \
     get_saml2_request_message_async_binding, create_saml2_server, \
-    get_saml2_metadata, get_sp_options_policy, get_idp_options_policy, \
+    get_saml2_metadata, get_sp_options_policy, \
     get_entity_id
 import authentic2.saml.saml2utils as saml2utils
-from authentic2.auth2_auth.models import AuthenticationEvent
+from authentic2.models import AuthenticationEvent
 from common import redirect_to_login, kill_django_sessions
-from authentic2.auth2_auth import NONCE_FIELD_NAME
-from authentic2.idp.interactions import consent_federation, consent_attributes
+from authentic2.constants import NONCE_FIELD_NAME
 
 from authentic2.idp import signals as idp_signals
 # from authentic2.idp.models import *
 
-from authentic2.authsaml2.models import SAML2TransientUser
-from authentic2.utils import cache_and_validate
+from authentic2.utils import (cache_and_validate, get_backends as
+        get_idp_backends, get_username)
+from authentic2.decorators import is_transient_user
+from authentic2.attributes_ng.engine import get_attributes
 
 logger = logging.getLogger('authentic2.idp.saml')
 
@@ -170,6 +171,25 @@ def fill_assertion(request, saml_request, assertion, provider_id, nid_format):
     assertion.subject.nameID.format = NAME_ID_FORMATS[nid_format]['samlv2']
 
 
+def add_attributes(assertion, provider, policy, ctx):
+    qs = SAMLAttribute.objects.for_generic_object(provider)
+    qs |= SAMLAttribute.objects.for_generic_object(policy)
+    qs = qs.distinct()
+
+    if not assertion.attributeStatement:
+        assertion.attributeStatement = [lasso.Saml2AttributeStatement()]
+    attribute_statement = assertion.attributeStatement[0]
+    saml_attributes = list(attribute_statement.attribute)
+    for definition in qs:
+        saml_attribute = definition.to_lasso_attribute(ctx)
+        if not saml_attribute:
+            continue
+        logger.debug('adding attribute %r with value %r',
+                definition.name, ctx.get(definition.attribute_name))
+        saml_attributes.append(saml_attribute)
+    attribute_statement.attribute = saml_attributes
+
+
 def saml2_add_attribute_values(assertion, attributes):
     if not attributes:
         logger.info("\
@@ -248,18 +268,9 @@ def build_assertion(request, login, nid_format='transient', attributes=None):
         logger.debug("authentication from session %s" \
             % backend)
         if backend in ('django.contrib.auth.backends.ModelBackend',
-                'authentic2.idp.auth_backends.LogginBackend',
+                'authentic2.backends.models_backend.ModelBackend',
                 'django_auth_ldap.backend.LDAPBackend'):
             authn_context = lasso.SAML2_AUTHN_CONTEXT_PASSWORD
-        elif backend == 'authentic2.auth2_auth.auth2_ssl.backend.SSLBackend':
-            authn_context = lasso.SAML2_AUTHN_CONTEXT_X509
-        # XXX: grab context from the assertion received
-        elif backend == \
-                'authentic2.authsaml2.backends.AuthSAML2PersistentBackend':
-            authn_context = lasso.SAML2_AUTHN_CONTEXT_UNSPECIFIED
-        elif backend == \
-                'authentic2.authsaml2.backends.AuthSAML2TransientBackend':
-            authn_context = lasso.SAML2_AUTHN_CONTEXT_UNSPECIFIED
         else:
             backend = load_backend(backend)
             if hasattr(backend, 'get_saml2_authn_context'):
@@ -332,6 +343,7 @@ def build_assertion(request, login, nid_format='transient', attributes=None):
     register_new_saml2_session(request, login)
 
 
+@never_cache
 @csrf_exempt
 def sso(request):
     """Endpoint for receiving saml2:AuthnRequests by POST, Redirect or SOAP.
@@ -375,10 +387,17 @@ def sso(request):
                 "answered because no valid protocol binding could be found")
             logger.error("the request cannot be answered because no "
                 "valid protocol binding could be found")
+            login.response.status.statusMessage = 'No valid protocol binding could be found'
             return HttpResponseBadRequest(message)
+        except lasso.ProviderMissingPublicKeyError, e:
+            log_info_authn_request_details(login)
+            logger.error('no public key found: %s', e)
+            login.response.status.statusMessage = 'The public key is unknown'
+            return return_login_response(request, login)
         except lasso.DsError, e:
             log_info_authn_request_details(login)
-            logger.error('digital signature treatment error: %s' % e)
+            logger.error('digital signature treatment error: %s', e)
+            login.response.status.statusMessage = 'Signature validation failed'
             return return_login_response(request, login)
         except (lasso.ServerProviderNotFoundError,
                 lasso.ProfileUnknownProviderError):
@@ -389,10 +408,13 @@ def sso(request):
             provider_loaded = load_provider(request, provider_id,
                     server=login.server, autoload=True)
             if not provider_loaded:
-                message = _('sso: fail to load unknown provider %s' \
-                    % provider_id)
-                return error_page(request, message, logger=logger,
-                        warning=True)
+                add_url = reverse('admin:saml_libertyprovider_add_from_url')
+                add_url += '?' + urllib.urlencode({ 'entity_id': provider_id })
+                return render(request,
+                        'idp/saml/unknown_provider.html',
+                        { 'entity_id': provider_id,
+                          'add_url': add_url,
+                        })
             else:
                 policy = get_sp_options_policy(provider_loaded)
                 if not policy:
@@ -427,7 +449,7 @@ def sso(request):
            default_nid_format != nid_format:
             set_saml2_response_responder_status_code(login.response,
                 lasso.SAML2_STATUS_CODE_INVALID_NAME_ID_POLICY)
-            logger.error('NameID format required is not accepted')
+            logger.warning('NameID format required is not accepted')
             return finish_sso(request, login)
     else:
         logger.debug('no nameID policy format')
@@ -467,7 +489,7 @@ def need_consent_for_federation(request, login, save, nid_format):
     if not display_name:
         display_name = urllib.quote(login.request.issuer.content)
     url = '%s?%s=%s&next=%s&provider_id=%s' \
-        % (reverse(consent_federation), NONCE_FIELD_NAME,
+        % (reverse('a2-consent-federation'), NONCE_FIELD_NAME,
             nonce, get_url_with_nonce(request, continue_sso, nonce),
             display_name)
     logger.debug('redirect to url %s' % url)
@@ -488,13 +510,14 @@ def need_consent_for_attributes(request, login, consent_obtained, save,
     if not display_name:
         display_name = urllib.quote(login.request.issuer.content)
     url = '%s?%s=%s&next=%s&provider_id=%s' \
-        % (reverse(consent_attributes), NONCE_FIELD_NAME,
+        % (reverse('a2-consent-attributes'), NONCE_FIELD_NAME,
             nonce, get_url_with_nonce(request, continue_sso, nonce),
             display_name)
     logger.debug('redirect to url %s' % url)
     return HttpResponseRedirect(url)
 
 
+@never_cache
 def continue_sso(request):
     consent_answer = None
     consent_attribute_answer = None
@@ -582,8 +605,7 @@ def sso_after_process_request(request, login, consent_obtained=False,
     #Deal with transient users
     transient_user = False
     # XXX: Deal with all kind of transient users
-    type(SAML2TransientUser)
-    if isinstance(request.user, SAML2TransientUser):
+    if is_transient_user(request.user):
         logger.debug('the user is transient')
         transient_user = True
     if transient_user and login.request.nameIdPolicy.format == \
@@ -818,6 +840,13 @@ def sso_after_process_request(request, login, consent_obtained=False,
 
     build_assertion(request, login, nid_format=nid_format,
             attributes=attributes)
+    ctx = get_attributes({
+        'request': request,
+        'user': request.user,
+        'provider': provider,
+        'policy': saml_policy,
+    })
+    add_attributes(login.assertion, provider, saml_policy, ctx)
     return finish_sso(request, login, user=user, save=save, return_profile=return_profile)
 
 
@@ -883,6 +912,7 @@ def reload_artifact(login):
         pass
 
 
+@never_cache
 @csrf_exempt
 def artifact(request):
     '''Resolve a SAMLv2 ArtifactResolve request
@@ -925,6 +955,7 @@ def check_delegated_authentication_permission(request):
     return request.user.is_superuser()
 
 
+@never_cache
 @csrf_exempt
 @login_required
 def idp_sso(request, provider_id=None, user_id=None, nid_format=None,
@@ -1008,6 +1039,7 @@ def idp_sso(request, provider_id=None, user_id=None, nid_format=None,
             nid_format=nid_format, return_profile=return_profile)
 
 
+@never_cache
 def finish_slo(request):
     id = request.REQUEST.get('id')
     if not id:
@@ -1114,7 +1146,7 @@ def validate_logout_request(request, logout, idp=True):
 
 
 def logout_synchronous_other_backends(request, logout, django_sessions_keys):
-    backends = idp.get_backends()
+    backends = get_idp_backends()
     if backends:
         logger.info('backends %s' \
             % str(backends))
@@ -1130,7 +1162,7 @@ def logout_synchronous_other_backends(request, logout, django_sessions_keys):
     return None
 
 
-def get_only_last_session(name_id, session_indexes, but_provider):
+def get_only_last_session(issuer_id, provider_id, name_id, session_indexes):
     """Try to have a decent behaviour when receiving a logout request with
        multiple session indexes.
 
@@ -1140,14 +1172,14 @@ def get_only_last_session(name_id, session_indexes, but_provider):
     logger.debug('%s %s' % (name_id.dump(),
         session_indexes))
     lib_session1 = LibertySession.get_for_nameid_and_session_indexes(
-            name_id, session_indexes)
+            issuer_id, provider_id, name_id, session_indexes)
     django_session_keys = [s.django_session_key for s in lib_session1]
     lib_session = LibertySession.objects.filter(
             django_session_key__in=django_session_keys)
     providers = set([s.provider_id for s in lib_session])
     result = []
     for provider in providers:
-        if provider != but_provider:
+        if provider != provider_id:
             x = lib_session.filter(provider_id=provider)
             latest = x.latest('creation')
             result.append(latest)
@@ -1188,6 +1220,7 @@ def set_session_dump_from_liberty_sessions(profile, lib_sessions):
         % profile.session.dump())
 
 
+@never_cache
 @csrf_exempt
 def slo_soap(request):
     """Endpoint for receiveing saml2:AuthnRequest by SOAP"""
@@ -1225,8 +1258,9 @@ def slo_soap(request):
 
     '''Find all active sessions on SPs but the SP initiating the SLO'''
     found, lib_sessions, django_session_keys = \
-            get_only_last_session(logout.request.nameId,
-                    logout.request.sessionIndexes, logout.remoteProviderId)
+            get_only_last_session(logout.server.providerId,
+                    logout.remoteProviderId, logout.request.nameId,
+                    logout.request.sessionIndexes)
     if not found:
         logger.debug('no third SP session found')
     else:
@@ -1286,52 +1320,6 @@ def slo_soap(request):
                 logger.exception('slo, relaying to %s failed ' %
                         lib_session.provider_id)
 
-    #Send SLO to IdP
-    pid = None
-    q = LibertySessionDump. \
-            objects.filter(django_session_key__in=django_session_keys,
-                    kind=LIBERTY_SESSION_DUMP_KIND_SP)
-    if not q:
-        logger.info('No session found for a third IdP')
-    else:
-        from authentic2.authsaml2 import saml2_endpoints
-        server = saml2_endpoints.create_server(request)
-        logout2 = lasso.Logout(server)
-        for s in q:
-            logger.debug('IdP session found %s' % s.session_dump)
-            try:
-                lib_session = lasso.Session().newFromDump(s.session_dump.encode('utf-8'))
-            except lasso.Error:
-                logger.debug('Unable to load session %s' % s.session_dump)
-            else:
-                try:
-                    pid = lib_session.get_assertions().keys()[0]
-                    logger.debug('SLO to %s' % pid)
-                    logout2.setSessionFromDump(s.session_dump.encode('utf8'))
-                    provider = load_provider(request, pid,
-                        server=server, sp_or_idp='idp')
-                    policy = get_idp_options_policy(provider)
-                    if not policy:
-                        logger.error('No policy found for %s'\
-                             % provider)
-                    elif not policy.forward_slo:
-                        logger.info('%s configured to not receive \
-                            slo' % provider)
-                    else:
-                        '''
-                            As we are in a synchronous binding,
-                            we need SOAP support
-                        '''
-                        logout2.initRequest(None, lasso.HTTP_METHOD_SOAP)
-                        logout2.buildRequestMsg()
-                        soap_response = send_soap_request(request, logout2)
-                        logout2.processResponseMsg(soap_response)
-                        logger.info('successful SLO with %s' \
-                            % pid)
-                except Exception, e:
-                    logger.error('error treating SLO with IdP %s' \
-                        % str(e))
-
     '''
         Respond to the SP initiating SLO
     '''
@@ -1348,6 +1336,7 @@ def slo_soap(request):
         title=_('You are being redirected to "%s"') % provider.name)
 
 
+@never_cache
 @csrf_exempt
 def slo(request):
     """Endpoint for receiving SLO by POST, Redirect.
@@ -1407,17 +1396,15 @@ def slo(request):
     logger.info('asynchronous slo from %s' % logout.remoteProviderId)
     # Filter sessions
     all_sessions = LibertySession.get_for_nameid_and_session_indexes(
+            logout.server.providerId, logout.remoteProviderId,
             logout.request.nameId, logout.request.sessionIndexes)
-    # Does the request is valid ?
-    remote_provider_sessions = \
-            all_sessions.filter(provider_id=logout.remoteProviderId)
-    if not remote_provider_sessions.exists():
+    if not all_sessions.exists():
         logger.error('slo refused, since no session exists with the \
             requesting provider')
         return return_logout_error(request, logout,
                 AUTHENTIC_STATUS_CODE_UNKNOWN_SESSION)
     # Load session dump for the requesting provider
-    last_session = remote_provider_sessions.latest('creation')
+    last_session = all_sessions.latest('creation')
     set_session_dump_from_liberty_sessions(logout, [last_session])
     try:
         logout.validateRequest()
@@ -1431,7 +1418,7 @@ def slo(request):
     # Save some values for cleaning up
     save_key_values(logout.request.id, logout.dump(),
             request.session.session_key)
-    return idp_views.redirect_to_logout(request, next_page='%s?id=%s' %
+    return a2_views.redirect_to_logout(request, next_page='%s?id=%s' %
             (reverse(finish_slo), urllib.quote(logout.request.id)))
 
 
@@ -1445,6 +1432,7 @@ def ok_icon(request):
         % settings.STATIC_URL)
 
 
+@never_cache
 @csrf_exempt
 @login_required
 def idp_slo(request, provider_id=None):
@@ -1495,7 +1483,7 @@ def idp_slo(request, provider_id=None):
         logger.debug('%d lib_sessions found', lib_sessions.count())
         set_session_dump_from_liberty_sessions(logout, [lib_sessions[0]])
     try:
-        logout.initRequest(provider_id)
+        logout.initRequest(provider_id, policy.http_method_for_slo_request)
     except (lasso.ProfileMissingAssertionError,
             lasso.ProfileSessionNotFoundError):
         logger.error('slo failed because no sessions exists for %r' \
@@ -1547,6 +1535,7 @@ def process_logout_response(request, logout, soap_response, next):
     return redirect_next(request, next) or ok_icon(request)
 
 
+@never_cache
 def slo_return(request):
     next = None
     logger.info('return from redirect')
