@@ -1,7 +1,17 @@
 # Django settings for authentic project.
 import os
-from django.core.exceptions import ImproperlyConfigured
 import json
+import glob
+import re
+
+from django.core.exceptions import ImproperlyConfigured
+
+from . import plugins
+
+try:
+    import yaml
+except ImportError:
+    yaml = None
 
 gettext_noop = lambda s: s
 
@@ -35,8 +45,11 @@ DEBUG_PROPAGATE_EXCEPTIONS = 'DEBUG_PROPAGATE_EXCEPTIONS' in os.environ
 USE_DEBUG_TOOLBAR = 'USE_DEBUG_TOOLBAR' in os.environ
 TEMPLATE_DEBUG = DEBUG
 
-PROJECT_PATH = os.path.join(os.path.dirname(__file__), '..')
+BASE_DIR = os.path.dirname(__file__)
+PROJECT_DIR = os.path.join(BASE_DIR, '..')
 PROJECT_NAME = 'authentic2'
+VAR_DIR = os.path.join('/var/lib/', PROJECT_NAME)
+ETC_DIR = os.path.join('/etc', PROJECT_NAME)
 
 ADMINS = ()
 if 'ADMINS' in os.environ:
@@ -52,7 +65,7 @@ MANAGERS = ADMINS
 DATABASES = {
     'default': {
         'ENGINE': 'django.db.backends.sqlite3',
-        'NAME': os.path.join(PROJECT_PATH, PROJECT_NAME + '.db'),
+        'NAME': os.path.join(PROJECT_DIR, PROJECT_NAME + '.db'),
     }
 }
 
@@ -64,23 +77,13 @@ for key in os.environ:
 # Hey Entr'ouvert is in France !!
 TIME_ZONE = 'Europe/Paris'
 LANGUAGE_CODE = 'fr'
-SITE_ID = 1
 USE_I18N = True
-
-LANGUAGES = (
-    ('en', gettext_noop('English')),
-    ('fr', gettext_noop('French')),
-)
 USE_L10N = True
 
 # Static files
 
-STATIC_ROOT = os.environ.get('STATIC_ROOT', '/var/lib/%s/static' % PROJECT_NAME)
+STATIC_ROOT = os.environ.get('STATIC_ROOT', os.path.join(VAR_DIR, 'static'))
 STATIC_URL = os.environ.get('STATIC_URL', '/static/')
-if 'STATICFILES_DIRS' in os.environ:
-    STATICFILES_DIRS = os.environ['STATICFILES_DIRS'].split(':')
-else:
-    STATICFILES_DIRS = ('/var/lib/%s/extra-static/' % PROJECT_NAME,)
 
 if DEBUG:
     TEMPLATE_LOADERS = (
@@ -102,10 +105,11 @@ TEMPLATE_CONTEXT_PROCESSORS = (
     'django.core.context_processors.request',
     'django.contrib.messages.context_processors.messages',
     'django.core.context_processors.static',
-    'authentic2.context_processors.federations_processor',
+    'authentic2.context_processors.a2_processor',
 )
 
 MIDDLEWARE_CLASSES = (
+    'authentic2.middleware.LoggingCollectorMiddleware',
     'django.middleware.common.CommonMiddleware',
     'django.middleware.http.ConditionalGetMiddleware',
     'django.contrib.sessions.middleware.SessionMiddleware',
@@ -114,19 +118,52 @@ MIDDLEWARE_CLASSES = (
     'django.contrib.auth.middleware.AuthenticationMiddleware',
     'django.contrib.messages.middleware.MessageMiddleware',
     'django.middleware.transaction.TransactionMiddleware',
-    'authentic2.idp.middleware.DebugMiddleware'
+    'authentic2.idp.middleware.DebugMiddleware',
+    'authentic2.middleware.CollectIPMiddleware',
 )
 
+A2_OPENED_SESSION_COOKIE_DOMAIN = os.environ.get('A2_OPENED_SESSION_COOKIE_DOMAIN')
+if A2_OPENED_SESSION_COOKIE_DOMAIN:
+    MIDDLEWARE_CLASSES += (
+        'authentic2.middleware.OpenedSessionCookieMiddleware',
+    )
+
+MIDDLEWARE_CLASSES = plugins.register_plugins_middleware(MIDDLEWARE_CLASSES)
+
 ROOT_URLCONF = 'authentic2.urls'
 
+TEMPLATE_DIRS = (
+        os.path.join(VAR_DIR, 'templates'),
+        os.path.join(BASE_DIR, 'templates'),
+)
+
+STATICFILES_DIRS = (
+        os.path.join(VAR_DIR, 'extra-static'),
+        os.path.join(BASE_DIR, 'static'),
+)
+
 if os.environ.get('TEMPLATE_DIRS'):
-    TEMPLATE_DIRS = os.environ['TEMPLATE_DIRS'].split(':')
-else:
-    TEMPLATE_DIRS = ('/var/lib/%s/templates' % PROJECT_NAME,)
+    TEMPLATE_DIRS = tuple(os.environ['TEMPLATE_DIRS'].split(':')) + TEMPLATE_DIRS
+
+TEMPLATE_VARS = {}
+
+if os.environ.get('STATICFILES_DIRS'):
+    STATICFILES_DIRS = tuple(os.environ['STATICFILES_DIRS'].split(':')) + STATICFILES_DIRS
 
 
 INSTALLED_APPS = (
-    'authentic2',
+    'django.contrib.staticfiles',
+    'django.contrib.auth',
+    'django.contrib.contenttypes',
+    'django.contrib.sessions',
+    'django.contrib.messages',
+    'south',
+    'admin_tools',
+    'admin_tools.theming',
+    'admin_tools.menu',
+    'admin_tools.dashboard',
+    'django.contrib.admin',
+    'registration',
     'authentic2.nonce',
     'authentic2.saml',
     'authentic2.idp',
@@ -134,29 +171,21 @@ INSTALLED_APPS = (
     'authentic2.auth2_auth',
     'authentic2.attribute_aggregator',
     'authentic2.disco_service',
-    'admin_tools',
-    'admin_tools.theming',
-    'admin_tools.menu',
-    'admin_tools.dashboard',
-    'django.contrib.staticfiles',
-    'django.contrib.auth',
-    'django.contrib.contenttypes',
-    'django.contrib.sessions',
-    'django.contrib.messages',
-    'django.contrib.admin',
-    'django.contrib.sites',
-    'registration',
-    'south',
+    'authentic2',
 )
 
+INSTALLED_APPS = plugins.register_plugins_installed_apps(INSTALLED_APPS)
+
 MESSAGE_STORAGE = 'django.contrib.messages.storage.session.SessionStorage'
 
 
 
 # authentication
 AUTHENTICATION_BACKENDS = (
-    'django.contrib.auth.backends.ModelBackend',
+    'authentic2.backends.ModelBackend',
 )
+AUTHENTICATION_BACKENDS = plugins.register_plugins_authentication_backends(
+        AUTHENTICATION_BACKENDS)
 
 # sessions
 SESSION_EXPIRE_AT_BROWSER_CLOSE =  'SESSION_EXPIRE_AT_BROWSER_CLOSE' in os.environ
@@ -181,6 +210,8 @@ DEFAULT_FROM_EMAIL = os.environ.get('DEFAULT_FROM_EMAIL', 'webmaster@localhost')
 if 'ALLOWED_HOSTS' in os.environ:
     ALLOWED_HOSTS = os.environ['ALLOWED_HOSTS'].split(':')
 USE_X_FORWARDED_HOST = 'USE_X_FORWARDED_HOST' in os.environ
+if 'SECURE_PROXY_SSL_HEADER' in os.environ:
+    SECURE_PROXY_SSL_HEADER = os.environ['SECURE_PROXY_SSL_HEADER'].split(':', 1)
 LOGIN_REDIRECT_URL = os.environ.get('LOGIN_REDIRECT_URL', '/')
 LOGIN_URL = os.environ.get('LOGIN_URL', '/login')
 LOGOUT_URL = os.environ.get('LOGOUT_URL', '/accounts/logout')
@@ -202,14 +233,14 @@ DISCO_USE_OF_METADATA = 'DISCO_USE_OF_METADATA' in os.environ
 DISCO_SERVICE_NAME = os.environ.get('DISCO_SERVICE_NAME', "http://www.identity-hub.com/disco_service/disco")
 DISCO_RETURN_ID_PARAM = "entityID"
 SHOW_DISCO_IN_MD = 'SHOW_DISCO_IN_MD' in os.environ
-USE_DISCO_SERVICE = 'USE_DISCO_SERVICE' in os.environ
 
 ###########################
 # Authentication settings
 ###########################
 
 # Only RSA private keys are currently supported
-AUTH_FRONTENDS = ( 'authentic2.auth2_auth.backend.LoginPasswordBackend',)
+AUTH_FRONTENDS = ( 'authentic2.auth_frontends.LoginPasswordBackend',)
+AUTH_FRONTENDS = plugins.register_plugins_auth_frontends(AUTH_FRONTENDS)
 SSLAUTH_CREATE_USER = 'SSLAUTH_CREATE_USER' in os.environ
 AUTHENTICATION_EVENT_EXPIRATION = int(os.environ.get('AUTHENTICATION_EVENT_EXPIRATION', 3600*24*7))
 
@@ -219,7 +250,7 @@ AUTHENTICATION_EVENT_EXPIRATION = int(os.environ.get('AUTHENTICATION_EVENT_EXPIR
 
 # List of IdP backends, mainly used to show available services in the homepage
 # of user, and to handle SLO for each protocols
-IDP_BACKENDS = [ ]
+IDP_BACKENDS = plugins.register_plugins_idp_backends(())
 
 # You MUST changes these keys, they are just for testing !
 LOCAL_METADATA_CACHE_TIMEOUT = int(os.environ.get('LOCAL_METADATA_CACHE_TIMEOUT', 600))
@@ -278,6 +309,9 @@ SAML_METADATA_AUTOLOAD = os.environ.get('SAML_METADATA_AUTOLOAD', 'none')
 
 PUSH_PROFILE_UPDATES = 'PUSH_PROFILE_UPDATES' in os.environ
 
+if 'PASSWORD_HASHERS' in os.environ:
+    PASSWORD_HASHERS = os.environ['PASSWORD_HASHERS'].split(':')
+
 ##################################
 # LDAP Configuration
 ##################################
@@ -291,14 +325,32 @@ else:
 ##################################
 # Cache configuration
 ##################################
+CACHE_DIR = os.path.join('/var/cache/', PROJECT_NAME)
+if not os.access(CACHE_DIR, os.W_OK):
+    CACHE_DIR = os.path.join(PROJECT_DIR, 'cache')
+    print 'Cannot access global cache path, using in project cache directory', CACHE_DIR
+    if not os.path.isdir(CACHE_DIR):
+        os.makedirs(CACHE_DIR)
+
 CACHES = {
     'default': {
-        'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
+        'BACKEND': 'django.core.cache.backends.filebased.FileBasedCache',
+        'LOCATION': CACHE_DIR,
     },
 }
 if 'CACHE_BACKEND' in os.environ:
     CACHES['default'] = json.loads(os.environ['CACHE_BACKEND'])
 
+if 'USE_MEMCACHED' in os.environ:
+    CACHES = {
+            'default': {
+                'BACKEND': 'django.core.cache.backends.memcached.MemcachedCache',
+                'LOCATION': '127.0.0.1:11211',
+                'KEY_PREFIX': 'authentic2',
+                }
+            }
+    SESSION_ENGINE = 'django.contrib.sessions.backends.cached_db'
+
 # Logging settings
 
 # add sentry handler if environment contains SENTRY_DSN
@@ -308,6 +360,7 @@ if 'SENTRY_DSN' in os.environ:
     except ImportError:
         raise ImproperlyConfigured('SENTRY_DSN environment variable is set but raven is not installed.')
     SENTRY_DSN = os.environ['SENTRY_DSN']
+
 else:
     SENTRY_DSN = None
 
@@ -318,22 +371,72 @@ SOUTH_TESTS_MIGRATE = False
 ############################
 A2_CAN_RESET_PASSWORD = to_boolean('A2_CAN_RESET_PASSWORD')
 A2_REGISTRATION_CAN_DELETE_ACCOUNT = to_boolean('A2_REGISTRATION_CAN_DELETE_ACCOUNT')
-A2_REGISTRATION_EMAIL_IS_UNIQUE = to_boolean('A2_REGISTRATION_EMAIL_IS_UNIQUE')
+A2_REGISTRATION_EMAIL_IS_UNIQUE = to_boolean('A2_REGISTRATION_EMAIL_IS_UNIQUE', default=False)
 REGISTRATION_OPEN = to_boolean('REGISTRATION_OPEN')
 ACCOUNT_ACTIVATION_DAYS = to_int('ACCOUNT_ACTIVATION_DAYS', 3)
 PASSWORD_RESET_TIMEOUT_DAYS = to_int('PASSWORD_RESET_TIMEOUT_DAYS', 3)
 
+if 'A2_HOMEPAGE_URL' in os.environ:
+    A2_HOMEPAGE_URL = os.environ['A2_HOMEPAGE_URL']
+
 # Admin tools
 ADMIN_TOOLS_INDEX_DASHBOARD = 'authentic2.dashboard.CustomIndexDashboard'
 ADMIN_TOOLS_APP_INDEX_DASHBOARD = 'authentic2.dashboard.CustomAppIndexDashboard'
 ADMIN_TOOLS_MENU = 'authentic2.menu.CustomMenu'
 
-AUTH_SAML2 = 'AUTH_SAML2' in os.environ
 AUTH_OPENID = 'AUTH_OPENID' in os.environ
 AUTH_SSL = 'AUTH_SSL' in os.environ
 IDP_SAML2 = 'IDP_SAML2' in os.environ
 IDP_OPENID = 'IDP_OPENID' in os.environ
-IDP_CAS = 'IDP_CAS' in os.environ
+
+# extract any key starting with setting
+for key in os.environ:
+    if key.startswith('SETTING_'):
+        setting_key = key[len('SETTING_'):]
+        value = os.environ[key]
+        try:
+             value = int(value)
+        except ValueError:
+             pass
+        globals()[setting_key] = value
+
+# Remove after Django 1.7
+SERIALIZATION_MODULES = {
+        'json': 'authentic2.serializers',
+}
+
+DEBUG_LOG = os.environ.get('DEBUG_LOG')
+
+def load_dict_config(d):
+    for key in d:
+        if re.match('^[A-Z][_A-Z0-9]*$', key):
+            globals()[key] = d[key]
+
+CONFIG_DIRS = [ETC_DIR]
+if 'CONFIG_DIRS' in os.environ:
+    CONFIG_DIRS += os.environ['CONFIG_DIRS'].split(':')
+
+if yaml:
+    for config_dir in CONFIG_DIRS:
+        wildcard = os.path.join(config_dir, '*.yaml')
+        for path in sorted(glob.glob(wildcard)):
+            yaml_config = yaml.load(file(path))
+            if not isinstance(yaml_config, dict):
+                raise ImproperlyConfigured('YAML file %r is not a dictionnary' % path)
+            load_dict_config(yaml_config)
+
+for config_dir in CONFIG_DIRS:
+    wildcard = os.path.join(config_dir, '*.json')
+    for path in sorted(glob.glob(wildcard)):
+        json_config = json.load(file(path))
+        if not isinstance(json_config, dict):
+            raise ImproperlyConfigured('JSON file %r is not a dictionnary' % path)
+        load_dict_config(json_config)
+
+for config_dir in CONFIG_DIRS:
+    config_py = os.path.join(config_dir, 'config.py')
+    if os.path.exists(config_py):
+        execfile(config_py, globals())
 
 try:
     from local_settings import *
@@ -341,9 +444,11 @@ except ImportError, e:
     if 'local_settings' in e.args[0]:
         pass
 
+from . import fix_user_model
+
 LOGGING = {
     'version': 1,
-    'disable_existing_loggers': True,
+    'disable_existing_loggers': False,
     'filters': {
         'cleaning': {
             '()':  'authentic2.utils.CleanLogMessage',
@@ -354,23 +459,27 @@ LOGGING = {
             'format': '[%(asctime)s] %(levelname)s %(name)s: %(message)s',
             'datefmt': '%Y-%m-%d %a %H:%M:%S'
         },
+        'syslog': {
+            'format': '%(levelname)s %(name)s: %(message)s',
+        },
     },
     'handlers': {
         'null': {
-            'level':'DEBUG',
+            'level': 'DEBUG',
             'class':'django.utils.log.NullHandler',
         },
         'console': {
-            'level':'DEBUG',
+            'level': 'DEBUG',
             'class':'logging.StreamHandler',
             'formatter': 'verbose',
             'filters': ['cleaning'],
         },
         'syslog': {
-            'level': 'DEBUG' if DEBUG else 'INFO',
+            'level': 'DEBUG',
             'address': '/dev/log',
-            'class':'logging.handlers.SysLogHandler',
+            'class': 'logging.handlers.SysLogHandler',
             'filters': ['cleaning'],
+            'formatter': 'syslog',
         },
         'mail_admins': {
             'level': 'ERROR',
@@ -386,23 +495,46 @@ LOGGING = {
             'propagate': True,
         },
         'django.db': {
-            'handlers': ['mail_admins', 'syslog'] + (['console'] if DEBUG else []),
-            'level': 'INFO',
-            'propagate': False,
+            'handlers': ['null'],
+            'propagate': True,
         },
         '': {
-                'handlers': ['mail_admins', 'syslog'] + (['console'] if DEBUG else []),
-                'level': 'DEBUG' if DEBUG else 'INFO',
+                'handlers': ['mail_admins', 'syslog'],
+                'level': 'INFO',
         }
     },
 }
 
+if DEBUG and not DEBUG_LOG:
+    LOGGING['loggers']['']['handlers'] += ['console']
+    LOGGING['loggers']['']['level'] = 'DEBUG'
+
+if DEBUG_LOG:
+    domains = DEBUG_LOG.split()
+    for domain in domains:
+        logger = LOGGING['loggers'].setdefault(domain, {
+                'handlers': ['mail_admins', 'syslog'],
+                'level': 'DEBUG',
+        })
+        if 'syslog' not in logger['handlers']:
+            logger['handlers'] += ['syslog']
+        if 'console' not in logger['handlers']:
+            logger['handlers'] += ['console']
+        logger['level'] = 'DEBUG'
+
 if SENTRY_DSN is not None:
+    try:
+        import raven
+    except ImportError:
+        raise ImproperlyConfigured('SENTRY_DSN present but raven is not installed')
+    RAVEN_CONFIG = {
+            'dsn': SENTRY_DSN,
+            }
+    INSTALLED_APPS += ('raven.contrib.django.raven_compat', )
     LOGGING['handlers']['sentry'] = {
             'level': 'ERROR',
-            'class': 'raven.handlers.logging.SentryHandler',
-            'dsn': SENTRY_DSN,
-    }
+            'class': 'raven.contrib.django.raven_compat.handlers.SentryHandler',
+            }
     LOGGING['loggers']['']['handlers'].append('sentry')
 
 if USE_DEBUG_TOOLBAR:
@@ -413,15 +545,6 @@ if USE_DEBUG_TOOLBAR:
     except ImportError:
         print "Debug toolbar missing, not loaded"
 
-if AUTH_SAML2:
-    INSTALLED_APPS += ('authentic2.authsaml2',)
-    AUTHENTICATION_BACKENDS += (
-            'authentic2.authsaml2.backends.AuthSAML2PersistentBackend',
-            'authentic2.authsaml2.backends.AuthSAML2TransientBackend')
-    AUTH_FRONTENDS += ('authentic2.authsaml2.frontend.AuthSAML2Frontend',)
-    IDP_BACKENDS += ('authentic2.authsaml2.backends.AuthSAML2Backend',)
-    DISPLAY_MESSAGE_ERROR_PAGE = True
-
 if AUTH_OPENID:
     INSTALLED_APPS += ('authentic2.auth2_auth.auth2_openid', 'django_authopenid',)
     AUTH_FRONTENDS += ('authentic2.auth2_auth.auth2_openid.backend.OpenIDFrontend',)
@@ -440,8 +563,8 @@ if IDP_OPENID:
     TEMPLATE_CONTEXT_PROCESSORS += ('authentic2.idp.idp_openid.context_processors.openid_meta',)
     # OPENID_ACTIONS = {"http://rp.example.com" : 'my-template.html', }
 
-if IDP_CAS:
-    INSTALLED_APPS += ('authentic2.idp.idp_cas',)
-
 if LDAP_AUTH_SETTINGS:
-    AUTHENTICATION_BACKENDS += ('authentic2.backends.LDAPBackend',)
+    AUTHENTICATION_BACKENDS = ('authentic2.backends.LDAPBackend',) + AUTHENTICATION_BACKENDS
+
+if DEBUG:
+    print 'Debugging mode is active'
